@@ -15,6 +15,9 @@ here = os.path.join(os.path.split(__file__)[0])
 from .base import TestExecutor, testharness_result_converter, reftest_result_converter
 from ..testrunner import Stop
 
+# Extra timeout to use after internal test timeout at which the harness
+# should force a timeout
+extra_timeout = 5 # seconds
 
 required_files = [("testharness_runner.html", "", False),
                   ("testharnessreport.js", "resources/", True)]
@@ -37,31 +40,27 @@ class MarionetteTestExecutor(TestExecutor):
         self.close_after_done = close_after_done
 
     def setup(self, runner):
-        """Connect to browser via marionette"""
+        """Connect to browser via Marionette."""
         self.runner = runner
 
         self.logger.debug("Connecting to marionette on port %i" % self.marionette_port)
         self.marionette = marionette.Marionette(host='localhost', port=self.marionette_port)
         # XXX Move this timeout somewhere
-        self.logger.debug("Waiting for marionette connection")
+        self.logger.debug("Waiting for Marionette connection")
         success = self.marionette.wait_for_port(60)
         session_started = False
         if success:
-            for i in xrange(5):
-                try:
-                    self.logger.debug("Starting marionette session attempt %i" % i)
-                    self.marionette.start_session()
-                except:
-                    self.logger.warning("Starting marionette session failed")
-                    time.sleep(1)
-                    break
-                else:
-                    self.logger.debug("Marionette session started")
-                    session_started = True
-                    break
+            try:
+                self.logger.debug("Starting Marionette session")
+                self.marionette.start_session()
+            except Exception as e:
+                self.logger.warning("Starting marionette session failed: %s" % e)
+            else:
+                self.logger.debug("Marionette session started")
+                session_started = True
 
         if not success or not session_started:
-            self.logger.warning("Failed to connect to marionette")
+            self.logger.warning("Failed to connect to Marionette")
             self.runner.send_message("init_failed")
         else:
             try:
@@ -84,20 +83,25 @@ class MarionetteTestExecutor(TestExecutor):
         try:
             # Get a simple property over the connection
             self.marionette.current_window_handle
-        except (socket.timeout, marionette.errors.InvalidResponseException):
+        except:
             return False
         return True
 
     def after_connect(self):
-        self.logger.debug(urlparse.urljoin(self.http_server_url, "/testharness_runner.html"))
+        url = urlparse.urljoin(
+            self.http_server_url, "/testharness_runner.html")
+        self.logger.debug(url)
         try:
-            self.marionette.navigate(
-                urlparse.urljoin(self.http_server_url, "/testharness_runner.html"))
+            self.marionette.navigate(url)
         except:
             self.logger.critical(
-                "Loading initial page %s failed. Ensure that the there are no other programs bound to this port and that your firewall rules or network setup does not prevent access.")
+                "Loading initial page %s failed. Ensure that the "
+                "there are no other programs bound to this port and "
+                "that your firewall rules or network setup does not "
+                "prevent access." % url)
             raise
-        self.marionette.execute_script("document.title = '%s'" % threading.current_thread().name)
+        self.marionette.execute_script(
+            "document.title = '%s'" % threading.current_thread().name)
 
     def run_test(self, test):
         """Run a single test.
@@ -122,14 +126,13 @@ class MarionetteTestExecutor(TestExecutor):
                     result = (test.result_cls("EXTERNAL-TIMEOUT", None), [])
                     self.runner.send_message("test_ended", test, result)
 
-        self.timer = threading.Timer(timeout + 10, timeout_func)
+        self.timer = threading.Timer(timeout + 2 * extra_timeout, timeout_func)
         self.timer.start()
 
         try:
-            self.marionette.set_script_timeout((timeout + 5) * 1000)
+            self.marionette.set_script_timeout((timeout + extra_timeout) * 1000)
         except IOError, marionette.errors.InvalidResponseException:
-            self.logger.error("Lost marionette connection")
-            self.runner.send_message("restart_test", test)
+            self.logger.error("Lost marionette connection before starting test")
             return Stop
 
         try:
@@ -179,6 +182,10 @@ class MarionetteTestharnessExecutor(MarionetteTestExecutor):
 
     def do_test(self, test, timeout):
         assert len(self.marionette.window_handles) == 1
+        if self.close_after_done:
+            self.logger.info("closing window")
+            self.marionette.execute_script("if (window.wrappedJSObject.win) {window.wrappedJSObject.win.close()}")
+
         return self.marionette.execute_async_script(
             self.script % {"abs_url": urlparse.urljoin(self.http_server_url, test.url),
                            "url": test.url,
@@ -186,11 +193,6 @@ class MarionetteTestharnessExecutor(MarionetteTestExecutor):
                            "timeout_multiplier": self.timeout_multiplier,
                            "timeout": timeout * 1000}, new_sandbox=False)
 
-    def after_result(self):
-        if self.close_after_done:
-            self.logger.info("closing window")
-            self.marionette.execute_script("window.wrappedJSObject.win.close()")
-
 
 class MarionetteReftestExecutor(MarionetteTestExecutor):
     convert_result = reftest_result_converter
