@@ -86,7 +86,6 @@ class TestRunner(object):
         the associated methods"""
         self.setup()
         commands = {"run_test": self.run_test,
-                    "after_result": self.after_result,
                     "stop": self.stop}
         while True:
             command, args = self.command_queue.get()
@@ -121,10 +120,6 @@ class TestRunner(object):
             self.logger.critical(traceback.format_exc())
             raise
 
-    def after_result(self):
-        self.executor.after_result()
-        self.send_message("test_done")
-
     def send_message(self, command, *args):
         self.result_queue.put((command, args))
 
@@ -164,7 +159,7 @@ class TestRunnerManager(threading.Thread):
     init_lock = threading.Lock()
 
     def __init__(self, suite_name, tests_queue, browser_cls, browser_kwargs,
-                 executor_cls, executor_kwargs, stop_flag):
+                 executor_cls, executor_kwargs, stop_flag, pause_on_unexpected=False):
         """Thread that owns a single TestRunner process and any processes required
         by the TestRunner (e.g. the Firefox binary).
 
@@ -193,6 +188,8 @@ class TestRunnerManager(threading.Thread):
         self.parent_stop_flag = stop_flag
         self.child_stop_flag = multiprocessing.Event()
 
+        self.pause_on_unexpected = pause_on_unexpected
+
         self.manager_number = get_manager_number()
 
         self.command_queue = Queue()
@@ -237,7 +234,6 @@ class TestRunnerManager(threading.Thread):
                                 "init_failed": self.init_failed,
                                 "test_start": self.test_start,
                                 "test_ended": self.test_ended,
-                                "test_done": self.test_done,
                                 "restart_runner": self.restart_runner,
                                 "runner_teardown": self.runner_teardown,
                                 "log": self.log,
@@ -323,9 +319,16 @@ class TestRunnerManager(threading.Thread):
                 self.browser.start()
                 self.start_test_runner()
             except:
+                self.logger.warning("Failure during init %s" % traceback.format_exc())
                 self.init_timer.cancel()
                 self.logger.error(traceback.format_exc())
-                self.init_failed()
+                succeeded = False
+            else:
+                succeeded = True
+
+        # This has to happen after the lock is released
+        if not succeeded:
+            self.init_failed()
 
     def init_succeeded(self):
         """Callback when we have started the browser, connected via
@@ -348,6 +351,9 @@ class TestRunnerManager(threading.Thread):
             return Stop
 
     def start_test_runner(self):
+        # Note that we need to be careful to start the browser before the
+        # test runner to ensure that any state set when the browser is started
+        # can be passed in to the test runner.
         assert self.command_queue is not None
         assert self.remote_queue is not None
         executor_browser_cls, executor_browser_kwargs = self.browser.executor_browser()
@@ -478,16 +484,16 @@ class TestRunnerManager(threading.Thread):
 
         self.test = None
 
+        if self.pause_on_unexpected and (subtest_unexpected or is_unexpected):
+            self.logger.info("Got an unexpected result, pausing until the browser exists")
+            self.browser.runner.process_handler.wait()
+
         # Handle starting the next test, with a runner restart if required
         if (file_result.status in ("CRASH", "EXTERNAL-TIMEOUT") or
             subtest_unexpected or is_unexpected):
             return self.restart_runner()
         else:
-            self.send_message("after_result")
-
-    def test_done(self):
-        self.logger.info("test_done")
-        return self.start_next_test()
+            return self.start_next_test()
 
     def restart_runner(self):
         """Stop and restart the TestRunner"""
@@ -507,7 +513,7 @@ class TestRunnerManager(threading.Thread):
 
 class ManagerGroup(object):
     def __init__(self, suite_name, size, browser_cls, browser_kwargs,
-                 executor_cls, executor_kwargs):
+                 executor_cls, executor_kwargs, pause_on_unexpected=False):
         """Main thread object that owns all the TestManager threads."""
         self.suite_name = suite_name
         self.size = size
@@ -519,6 +525,7 @@ class ManagerGroup(object):
         # Event that is polled by threads so that they can gracefully exit in the face
         # of sigint
         self.stop_flag = threading.Event()
+        self.pause_on_unexpected = pause_on_unexpected
         self.logger = structuredlog.StructuredLogger(suite_name)
 
     def __enter__(self):
@@ -538,7 +545,8 @@ class ManagerGroup(object):
                                         self.browser_kwargs,
                                         self.executor_cls,
                                         self.executor_kwargs,
-                                        self.stop_flag)
+                                        self.stop_flag,
+                                        self.pause_on_unexpected)
             manager.start()
             self.pool.add(manager)
 
