@@ -2,8 +2,9 @@ import unittest
 import sys
 sys.path.insert(0, "..")
 
-import wptrunner
+from wptrunner import wptrunner
 
+print wptrunner
 
 class MockTest(object):
     def __init__(self, id, timeout=10):
@@ -25,6 +26,8 @@ class TestEqualTimeChunker(unittest.TestCase):
     def test_include_all(self):
         tests = make_mock_manifest(("a", 10), ("a/b", 10), ("c", 10))
 
+        print len(tests)
+
         chunk_1 = list(wptrunner.EqualTimeChunker(3, 1)(tests))
         chunk_2 = list(wptrunner.EqualTimeChunker(3, 2)(tests))
         chunk_3 = list(wptrunner.EqualTimeChunker(3, 3)(tests))
