@@ -225,7 +225,7 @@ class GitTree(object):
         if prefix is None:
             args = ("-a",)
         else:
-            args = (prefix,)
+            args = ("--no-ignore-removal", prefix)
         self.git("add", *args)
 
     def create_patch(self, patch_name, message):
@@ -380,7 +380,14 @@ def update_metadata(config, paths, local_tree, wpt, initial_rev, bug):
             needs_human = metadata.update_expected(paths["sync"],
                                                    paths["metadata"],
                                                    log_files,
-                                                   rev_old=initial_rev)
+                                                   rev_old=initial_rev,
+                                                   ignore_existing=config["command-args"]["ignore_existing"])
+
+            if needs_human:
+                print >> sys.stderr, "The following files got updated metadata, but did not change in the test update:"
+                for item in needs_human:
+                    print >> sys.stderr, item
+
             if not local_tree.is_clean():
                 local_tree.add_new(os.path.relpath(paths["metadata"], local_tree.root))
                 local_tree.update_patch(include=[paths["metadata"]])
