@@ -2,17 +2,10 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+import argparse
 import os
 import sys
 
-import argparse
-
-from mozlog.structured import commandline
-
-import products
-
-root = os.path.join(os.path.split(__file__)[0])
-
 
 def abs_path(path):
     return os.path.abspath(os.path.expanduser(path))
@@ -34,6 +27,9 @@ def require_arg(kwargs, name, value_func=None):
 
 
 def create_parser(allow_mandatory=True):
+    from mozlog.structured import commandline
+    import browsers
+
     if not allow_mandatory:
         prefix = "--"
     else:
@@ -86,9 +82,15 @@ def create_parser(allow_mandatory=True):
     parser.add_argument("--no-capture-stdio", action="store_true", default=False,
                         help="Don't capture stdio and write to logging")
 
-    parser.add_argument("--product", action="store", choices=[item[0] for item in products.iter_products()],
+    parser.add_argument("--product", action="store", choices=browsers.product_list,
                         default="firefox")
 
+    parser.add_argument('--debugger',
+                        help="run under a debugger, e.g. gdb or valgrind")
+    parser.add_argument('--debugger-args', help="arguments to the debugger")
+    parser.add_argument('--pause-on-unexpected', action="store_true",
+                        help="Halt the test runner when an unexpected result is encountered")
+
     parser.add_argument("--b2g-no-backup", action="store_true", default=False,
                         help="Don't backup device before testrun with --product=b2g")
 
@@ -97,12 +99,26 @@ def create_parser(allow_mandatory=True):
 
 
 def check_args(kwargs):
+    from mozrunner import cli
+
     if kwargs["this_chunk"] > 1:
         require_arg(kwargs, "total_chunks", lambda x: x >= kwargs["this_chunk"])
 
         if kwargs["chunk_type"] == "none":
             kwargs["chunk_type"] = "equal_time"
 
+    if kwargs["debugger"] is not None:
+        debug_args, interactive = cli.debugger_arguments(kwargs["debugger"],
+                                                         kwargs["debugger_args"])
+        if interactive:
+            require_arg(kwargs, "processes", lambda x: x == 1)
+            kwargs["no_capture_stdio"] = True
+        kwargs["interactive"] = interactive
+        kwargs["debug_args"] = debug_args
+    else:
+        kwargs["interactive"] = False
+        kwargs["debug_args"] = None
+
     return kwargs
 
 
@@ -126,6 +142,7 @@ def create_parser_update(allow_mandatory=True):
     parser.add_argument("--update-expected-type", action="store", dest="run_type",
                         choices=["none", "try", "logfile"],
                         default="none", help="Process to use for updating the expectation data")
+    parser.add_argument("--ignore-existing", action="store_true", help="When updating test results only consider results from the logfiles provided, not existing expectations.")
     # Should make this required iff run=logfile
     parser.add_argument("run_log", nargs="*", type=abs_path,
                         help="Log file from run of tests")
